#!/bin/sh
#------------------------------------------------------------------------------
# /srv/www/admin/ow.cgi - httpd Frontend                                2.0.0
#
# Creation:      04.02.2009 kmw <news4kmw@web.de>
# Last Update:   21.06.2009 kmw
# Modification:  22.06.2009 Tiger
#------------------------------------------------------------------------------

. /srv/www/include/cgi-helper
: ${OW_OPT_CONF_FILE:='/var/run/ow.conf'}
. $OW_OPT_CONF_FILE
[ -f $OW_DEVICE_LIB ] && . $OW_DEVICE_LIB

# The following routines provide the structure for all metadata.
# The data itself is inserted through the refresh routine.

StatusLED ()
  {
  sl_id=$1
  sl_status=$2
  #--------------
  if [ -n "$sl_id" ]
  then
    echo '<img id="led-'$sl_id'" src="../img/ow-led-grey.gif">'
  fi
  }

Text ()
  {
  t_id=$1
  shift
  t_text=$*
  #--------------
  if [ -n "$t_id" ]
  then
    echo '<input id="txt-'$t_id'" class=ow_unboxed type=text readonly value="">'
  fi
  }

InputField ()
  {
  if_id=$1
  if_value=$2
  if_unit=$3
  #--------------
  if [ -n "$if_id" ]
  then
    echo '<input type=hidden name=command value="'$if_id'">'\
         '<input id="ifd-'$if_id'" type=text name=value value="">'"$if_unit"
  fi
  }

Command ()
  {
  sw_id=$1
  sw_command=$2
  #--------------
  if [ -n "$sw_id" -a -n "$sw_command" ]
  then
    echo '<input id="cmd-'$sw_id'" name=command type=hidden value="">'\
         '<input id="img-'$sw_id'" name=image type=image src="../img/ow-toggle-empty.gif" class=ow_unboxed>'
  fi
  }

# Command button execution

ParseCommand ()
  {
  pc_command=$1
  #--------------
  if [ -n "$pc_command" ]
  then
    if [ -n "`CheckUserRight admin`" -o -n "`CheckUserRight exec`" ]
    then
      eval `echo "$pc_command" | sed -n 's|^\([0-9A-Fa-f]\{2\}\)\([0-9A-Fa-f]\{12\}\)[0-9A-Fa-f]\{2\}-\([^-]*\)-\(.*\)$|pc_file='$OW_OWFS_PATH'/\1.\2/\3;pc_value=\4|p'`
      if [ -n "$pc_file" -a -f $pc_file ]; then echo -n $pc_value > $pc_file; fi
    fi
  fi
  }

# Basic interface routines

CheckUserRight()
  {
  cur_required_right=$1
  #--------------
  cur_result=""
  if [ -n "$REMOTE_USER" ]
  then
    cur_rights="`grep \"^$REMOTE_USER=\" /etc/httpd/rights | cut -d= -f2`"
    for cur_right in $cur_rights
    do
      case "`echo $cur_right | sed -n 's|^\([^:]*\):.*$|\1|p'`" in
        "$OW_RIGHTS_SECTION")
          cur_localrights="`echo $cur_right | sed 's|.*:||'`"
          if [ -z "$cur_localrights" ]; then cur_localrights="none"; fi
          ;;
        "") cur_globalrights=$cur_right;;
      esac
    done
    cur_rights=$cur_localrights
    if [ -z "$cur_rights" ]; then cur_rights=$cur_globalrights; fi
    if [ -z "$cur_rights" ]; then cur_rights="all"; fi
    for cur_right in `echo $cur_rights | tr ',' ' '`
    do
      case $cur_right in
        all) cur_result="yes";;
        $cur_required_right) cur_result="yes";;
      esac
    done
  else
    cur_result="yes"
  fi
  echo $cur_result
  }

ReadDeviceTree ()
  {
  ls -1 $OW_OWFS_PATH | sed -n '\|^[0-9A-Fa-f]\{2\}\.[0-9A-Fa-f]\{12\}$|p' | while read rdt_device
  do
    rdt_device_type=`cat $OW_OWFS_PATH/$rdt_device/type`
    if type OWDevice_$rdt_device_type > /dev/null
    then
      OWDevice_$rdt_device_type "$OW_OWFS_PATH/$rdt_device" | sed 's|.*|/'"$rdt_device"'/&|'
    else
      echo "/$rdt_device/$rdt_device_type : '<div class=ow_error>'$_ow_error_unknown_device_type'</div>'"
    fi
  done
  }

AliasDeviceTree ()
  {
  if [ -f $OW_ALIAS_FILE ]
  then
    adt_aliases=`cat $OW_ALIAS_FILE`
  else
    adt_aliases='s|.*|&|'
  fi
  while read line
  do
    echo $line | sed '{'"$adt_aliases"'}; \|![^/]* : |d'
  done
  }

ShowDeviceTree ()
  {
  sdt_group=$1
  sdt_action=$2
  #--------------
  echo '<table class="normtable">'
    echo "<tr><th id=ow_column_group>$_ow_header_group</th><th id=ow_column_data>$_ow_header_data</th></tr>"
    prev_group=""
    linecoloring=""
    if [ -z "$sdt_group" ]; then sdt_group="/"; fi
    s_group="`echo \"$sdt_group\" | sed 's|/|\\\/|g;s|\&|\\\&|g;s|\*|\\\*|g;s|\.|\\\.|g;s|\[|\\\[|g;s|\]|\\\]|g;s|\^|\\\^|g'`"
    s_grouproot="`echo \"$sdt_group\" | sed 's|[^/]*$||;h;s|^/\([^/]*\).*|/\1/|;s|/|\\\/|g;x;s|^/[^/]*/\(.*\)$|\1|;s|[^/]*/|[^\\\/]*[\\\/]*|g;x;G;s|\n||'`"
    # sed:
    #   Entries starting with current selection only
    #     Print if current selection is a property ..
    #     .. else ..
    #       Print if current selection contains a property ..
    #       .. else it's a subgroup, so add a " to make it stick at the top of the list during sort
    #   Entries not starting with current selection: List root group only
    #   Delete everything else
    # echo: Add a line for each subgroup of the current selection, as to guarantee, there is at least one matching entry in the list. This is needed for browsing through subgroups.
    # sort: case-insensive and ignore any special characters
    # sed:  Remove any preciding " in subgroup names of the current selection, as they were added for sorting only (see above)
    # sed
    #   Delete duplicate lines. Now we're dealing with the content of the current selection and the roots of all other groups, only.
    # sed
    #   Write meta-data to file,
    # while read
    #   Execute meta-data
    # sed
    #   Duplicate line, strip data, replace irrelevant subgroup names with undeletable white space and store white space and property name in separate lines
    #   Line #1: /root/subgroup/property : data
    #   Line #2: wsp wsp
    #   Line #3: property
    # sed
    #   Duplicate first line and put copy as url-encoded group in separate line
    #   Line #1: /root/subgroup/property : data
    #   Line #2: wsp wsp
    #   Line #3: property
    #   Line #4: url-encoded group
    # sed
    #   If first line contains a data delimiter (therefore data), HTML-encode entire set of lines as property
    # sed
    #   If line isn't allready HTML-encoded ..
    #     If property is a subgroup of the current selection, HTML-encode entire set of lines as subgroup
    #     Else HTML-encode set of lines as subgroup pointing to the current selection
    # sed: Insert line coloring to every second line
    ReadDeviceTree | AliasDeviceTree | ( \
      sed '\
        /^'"$s_group"'/ {\
          /^'"$s_group"'[ ]* : / P;\
          /^'"$s_group"'[ ]* : / !{\
            /^'"$s_group"'[^\/]* : / P;\
            /^'"$s_group"'[^\/]* : / !{s|^'"$s_group"'\([^/]*/\).*|'"$s_group"'"\1|p}}};\
        /^'"$s_group"'/ !{s|^\(/[^/]*[/]\).*|\1|p};\
        D' && \
      echo -n "$sdt_group" | sed 's|/\(.*\)/[^/]*|\1|;s|/|\n|g' | sed -n -e '/^$/ !{x;G;s|\n|/|p;x}' | sed 's|.*|&/|' ) |
      sort -f -d |
      sed 's|^'"$s_group"'"\([^/]*/\).*|'"$s_group"'\1|p' |
      sed '$!N;\
        /^\(.*\)\n\1$/!P;\
        D' |
      sed '/ : / w '"$OW_REFRESH_TEMP" |
      while read line; do eval echo $line; done |
      sed 'h;\
        s|[ ]* : .*||;\
        s|\([^/]*\)[/]*$|\n\1|;\
        s|^/||;\
        s|[^/]*/|\&nbsp\&nbsp\&nbsp|g;\
        x;G' |
      sed 'h;\
        s|[ ]* : .*||;\
        s/%/%25/g;s/ /%20/g;s/!/%21/g;s/"/%22/g;s/#/%23/g;s/\$/%24/g;s/\&/%26/g;s/'\''/%27/g;s/(/%28/g;s/)/%29/g;s/\*/%2a/g;s/+/%2b/g;s/,/%2c/g;s/-/%2d/g;s/\./%2e/g;s/\//%2f/g;s/:/%3a/g;s/;/%3b/g;s/</%3c/g;s/\=/%3d/g;s/>/%3e/g;s/?/%3f/g;s/@/%40/g;s/\[/%5b/g;s/\\/%5c/g;s/\]/%5d/g;s/\^/%5e/g;s/_/%5f/g;s/`/%60/g;s/{/%7b/g;s/|/%7c/g;s/}/%7d/g;s/~/%7e/g;s//%f6/g;s//%fc/g;s//%e4/g;s//%d6/g;s//%dc/g;s//%c4/g;\
        x;N;N;G' |
      sed '\
        / : / !{N;N;N}\
        / : / {N;N;N;s|\(.*[^ ]\) [ ]*: \(.*[^ ]\).*\n\(.*\)\n\(.*\)\n\(.*\)|<td>\3<img class=ow_bullet src=\"../img/ow-port.gif\">\4</td><td><form action="'"$myname"'" method="POST"><input type=hidden name=action value="'"$FORM_action"'"><input type=hidden name=group value="'"$FORM_group"'">\2</form></td></tr>|}' |
      sed '/^<td>/ !{\
          /^'"$s_grouproot"'$/ !{N;N;N;\
            s|\(.*[^ ]\).*\n\(.*\)\n\(.*\)\n\(.*\)|<td>\2<a href="'"$myname"'?action='"$sdt_action"'\&group=\4"><img class=ow_bullet src=\"../img/ow-plus.gif\">\3</a></td><td></td></tr>|}
          /^'"$s_grouproot"'$/ {N;N;N;\
              s|\(.*[^ ]\).*\n\(.*\)\n\(.*\)\n\(.*\)|<td>\2<a href="'"$myname"'?action='"$sdt_action"'\&group=\4"><img class=ow_bullet src=\"../img/ow-reload.gif\"><b>\3</b></a></td><td></td></tr>|}}' |
      sed '$!N; s|\(.*\)\n\(.*\)|<tr>\1\n<tr class=ow_linecoloring>\2|' |
      while read line; do echo "$line"; [ 1 = 2 ]; done &&
        echo "<tr><td colspan=2><b>$_ow_error_empty_tree</b></td></tr>" # || echo '<tr><th>'"$_ow_header_refresh_time"'</th><th><input id="RefreshTime" class=ow_unboxed type=text readonly value=""></th></tr>'
  echo "</table>"
  # Straighten metadata file
  cat $OW_REFRESH_TEMP | sed -n '/\$(/ s|.* : ||p' | sed 's|^[^$]*||;s|[^)]*$||;/^$/ !s|^|echo |' > $OW_REFRESH_TEMP.new
  if [ -s $OW_REFRESH_TEMP.new ]
  then
    (echo "#!/bin/sh" && cat $OW_REFRESH_TEMP.new) > $OW_REFRESH_TEMP
  else
    echo -n > $OW_REFRESH_TEMP
  fi
  rm $OW_REFRESH_TEMP.new
  chmod 555 $OW_REFRESH_TEMP
  }

ModuleStarted ()
  {
  ms_module=$1
  #--------------
  if [ -n "$ms_module" ]
  then
    eval ms_pid_file='$OW_'$ms_module'_PID_FILE'
    if [ -n "$ms_pid_file" -a -s $ms_pid_file ]
    then
      eval ms_task='$OW_'$ms_module'_BINARY'
      ms_task=`echo $ms_task | sed 's|.*/||;s| .*||'`
      ps w| grep "^[ ]*$(cat $ms_pid_file).*$ms_task .*"
    fi
  fi
  }

TaskStarted ()
  {
  ts_file=$1
  #--------------
  if [ -n "$ts_file" ]
  then
    ps w| grep " $ts_file " | grep -v " grep "
  fi
  }

ShowAdmin ()
  {
  sa_command=$1
  sa_action=$2
  #--------------
  case $sa_command in
    owfs_*)    sa_module=OWFS;;
    owshell_*) sa_module=OWSHELL;;
    owhttpd_*) sa_module=OWHTTPD;;
    uscript_*) sa_module=uscript;;
    *)         sa_module="";;
  esac
  if [ -n "$sa_module" ]
  then
    case $sa_command in
      owfs_stop|owshell_stop|owhttpd_stop)
        eval sa_task='$OW_'$sa_module'_BINARY'
        sa_task=`echo $sa_task | sed 's|.*/||;s| .*||'`
        if [ -n "`ModuleStarted $sa_module`" -a -n "$sa_task" ]
        then
          killall -SIGKILL $sa_task
          if [ $sa_command = owfs_stop ]
          then
            sleep 1
            umount $OW_OWFS_PATH
          fi
        fi
      ;;
      owfs_start|owshell_start|owhttpd_start)
        if [ -z "`ModuleStarted $sa_module`" ]
        then
          eval '$OW_'$sa_module'_BINARY -c $OW_MODULE_CONF_FILE'
        fi
      ;;
      uscript_start)
        if [ -x "$OW_USER_SCRIPT" -a -x "$OW_SCRIPT_WRAPPER" -a -z "`TaskStarted $OW_SCRIPT_WRAPPER`" ]
        then
          if [ -f "$OW_USER_SCRIPT_STOP" ]; then rm $OW_USER_SCRIPT_STOP; fi
          /bin/sh $OW_SCRIPT_WRAPPER &
        fi
      ;;
      uscript_stop)
        if [ -n "$OW_USER_SCRIPT_STOP" ]
        then
          echo -n > $OW_USER_SCRIPT_STOP
          sleep 1 # Wait for next user script cycle
        fi
      ;;
    esac
  fi
  echo '<table class="normtable">'
  for sa_module in OWFS OWSHELL OWHTTPD
  do
    if [ "`eval echo '$OW_'$sa_module`" = yes ]
    then
	    eval sa_fake='$OW_'$sa_module'_FAKE'
	    eval sa_tester='$OW_'$sa_module'_TESTER'
	    echo -n
      sa_button='<a href="'"$myname"'?action='"$sa_action"'&command='`echo $sa_module'_'|tr "A-Z" "a-z"`
      if [ -n "`ModuleStarted $sa_module`" ]
      then
        sa_button=$sa_button'stop"><img src="../img/ow-toggle-down.gif"></a>'
        if [ -z "$sa_fake" -a -z "$sa_tester" ]
        then
          sa_led=green
        else
          sa_led=yellow
        fi
      else
        sa_button=$sa_button'start"><img src="../img/ow-toggle-up.gif"></a>'
        sa_led=red
      fi
	    if [ -z "$sa_fake" ]
	    then
	      if [ -z "$sa_tester" ]
	      then
	        eval sa_mode='$OW_'$sa_module'_DEV'
	      else
	        sa_mode=$_ow_tester_mode
	      fi
	    else
	      sa_mode=$_ow_fake_mode
	    fi
	    echo '<tr><th>'$sa_module'</th><td><img src="../img/ow-led-'$sa_led'.gif">'$sa_button'</td><td>'"$sa_mode"'</td></tr>'
    fi
  done
  if [ -n "$OW_USER_SCRIPT" ]
  then
    sa_button='<a href="'"$myname"'?action='"$sa_action"'&command=uscript_'
    if [ -n "`TaskStarted $OW_SCRIPT_WRAPPER`" ]
    then
      if [ -n "`ModuleStarted OWFS`" ]
      then
        sa_led=green
      else
        sa_led=yellow
      fi
      sa_button=$sa_button'stop"><img src="../img/ow-toggle-down.gif"></a>'
    else
      sa_led=red
      sa_button=$sa_button'start"><img src="../img/ow-toggle-up.gif"></a>'
    fi
  else
    sa_led=grey
    sa_button=
  fi
  echo '<tr><th>'$_ow_header_user_script'</th><td><img src="../img/ow-led-'$sa_led'.gif">'$sa_button'</td><td>'$OW_USER_SCRIPT'</td></tr>'
  echo '</table>'
  }

: ${FORM_action:=status}

# Check User Access Right
case $FORM_action in
  status) check_rights "$OW_RIGHTS_SECTION" "view";;
  *)      check_rights "$OW_RIGHTS_SECTION" "admin";;
esac

show_html_header "$OW_OPT_TITLE" "$OW_CSS_FILE"
  _status=$_ow_tab_status
  _admin=$_ow_tab_admin
  if [ -n "`CheckUserRight admin`" ]
  then
    show_tab_header "$_status" "$myname?action=status" "$_admin" "$myname?action=admin"
  else
    show_tab_header "$_status" "$myname?action=status"
  fi
  case $FORM_action in
    status)
      command=`echo "$FORM_command-$FORM_value" | sed 's|-$||'`
      ParseCommand "$command"
      ShowDeviceTree "$FORM_group" "$FORM_action"
    ;;
    admin)
      ShowAdmin "$FORM_command" "$FORM_action"
    ;;
    debug)
      ReadDeviceTree | sed 's|$|<br>|'
    ;;
    *) show_error "$_ow_title_error" "$_ow_error_unknown_action";;
  esac
show_tab_footer

case $FORM_action in
  status)
    if [ -s $OW_REFRESH_TEMP ]
    then
      echo '<div id="refresh"><iframe src="'$OW_REFRESH_FILE'">Input State</iframe></div>'
    fi
  ;;
esac

show_html_footer
